package com.autentia.workflow;

import java.util.List;

import org.junit.Assert;
import org.junit.Test;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.autentia.workflow.errors.MyAppException;

/**
 * Test de workflow
 */
public class WorkflowTest {

	private static String[] files = new String[] { "applicationContext.xml" };

	final ApplicationContext context;

	public WorkflowTest() {
		this.context = new ClassPathXmlApplicationContext(WorkflowTest.files);
	}

	private void crearPedidos() throws MyAppException {
		final CommissioningMgr commissioningMgr = (CommissioningMgr)this.context.getBean("commissioningMgr");
		// creamos un nuevo pedido con el que trabajar
		Commissioning commissioning = commissioningMgr.newCommissioning();
		commissioning.setName("commissioning1");
		commissioning.setDescription("desc1");
		// al guardarlo se creara el workflow
		commissioningMgr.persist(commissioning);
		// vemos las acciones disponibles
		int[] workflowActions = commissioningMgr.getAvalaibleActions(commissioning);
		// slo debemos tener la accion de comprobar stock
		Assert.assertEquals(1, workflowActions.length);
		// la nica accin debe ser preparar el pedido (id=1)
		Assert.assertEquals(1, workflowActions[0]);

		// creamos otronuevo pedido con el que trabaja
		commissioning = commissioningMgr.newCommissioning();
		// al guardarlo se crea el workflow
		commissioningMgr.persist(commissioning);
		// vemos las acciones disponibles
		workflowActions = commissioningMgr.getAvalaibleActions(commissioning);
		// slo debemos tener la accion de comprobar stock
		Assert.assertEquals(1, workflowActions.length);
		// la nica accin debe ser preparar el pedido (id=1)
		Assert.assertEquals(1, workflowActions[0]);

		// comprobamos que tenemos 2 pedidos en la base de datos
		final List<Commissioning> commissionings = commissioningMgr.getCommissionings();
		Assert.assertEquals(2, commissionings.size());
	}

	/**
	 * Crea dos pedidos y recupera el primer pedido y ejecuta todo el workflow hasta el final
	 */
	@Test
	public void testConStock() {
		try {
			this.crearPedidos();
			final CommissioningMgr commissioningMgr = (CommissioningMgr)this.context.getBean("commissioningMgr");
			// creamos un nuevo pedido con el que trabajar
			final Commissioning commissioning = commissioningMgr.findById(1);
			// se debe haber recuperado la instancia
			Assert.assertNotNull(commissioning);
			// vemos las acciones disponibles
			int[] workflowActions = commissioningMgr.getAvalaibleActions(commissioning);
			// slo debemos tener la accion de preparar
			Assert.assertEquals(1, workflowActions.length);
			// la nica accin debe ser preparar el pedido (id=1)
			Assert.assertEquals(1, workflowActions[0]);

			// ejecutamos la accion
			commissioningMgr.executeAction(commissioning, workflowActions[0]);
			// como estamos con el pedido con id=1
			// debe pasar la comprobacin de stock, por lo
			// que estaremos en el paso con id=5
			Assert.assertEquals(5, commissioning.getState().intValue());
			// y si ahora recuperamos las acciones disponibles
			// slo debemos tener una, la accin de empaquetar (id=5)
			workflowActions = commissioningMgr.getAvalaibleActions(commissioning);
			Assert.assertEquals(1, workflowActions.length);
			Assert.assertEquals(5, workflowActions[0]);

			// ejecutamos la accin disponible es empaquetar (id=5)
			commissioningMgr.executeAction(commissioning, workflowActions[0]);
			// hemos pasado al paso 6
			Assert.assertEquals(6, commissioning.getState().intValue());

			// si ahora recuperamos las acciones disponibles
			// slo debemos tener una, la accin de enviar (id=6)
			workflowActions = commissioningMgr.getAvalaibleActions(commissioning);
			Assert.assertEquals(1, workflowActions.length);
			Assert.assertEquals(6, workflowActions[0]);

			// ejecutamos la accin disponible es empaquetar (id=5)
			commissioningMgr.executeAction(commissioning, workflowActions[0]);
			// hemos pasado al paso 7
			Assert.assertEquals(7, commissioning.getState().intValue());

			// ahora no debemos tener ninguna accin disponible al haber llegado
			// al final del workflow
			workflowActions = commissioningMgr.getAvalaibleActions(commissioning);
			Assert.assertEquals(0, workflowActions.length);

		} catch (final MyAppException e) {
			e.printStackTrace();
			Assert.fail("Error");
		}

	}

	/**
	 * Crea dos pedidos y recupera el segundo pedido, se ejecuta el workflow por la rama de "sin stock", se solicita el
	 * suministro, y en la segunda vuelta del workflow se cancela
	 */
	@Test
	public void testSinStock() {
		try {
			this.crearPedidos();
			final CommissioningMgr commissioningMgr = (CommissioningMgr)this.context.getBean("commissioningMgr");
			// creamos un nuevo pedido con el que trabajar
			final Commissioning commissioning = commissioningMgr.findById(2);
			// se debe haber recuperado la instancia
			Assert.assertNotNull(commissioning);
			// vemos las acciones disponibles
			int[] workflowActions = commissioningMgr.getAvalaibleActions(commissioning);
			// slo debemos tener la accion de preparar
			Assert.assertEquals(1, workflowActions.length);
			// la nica accin debe ser preparar el pedido (id=1)
			Assert.assertEquals(1, workflowActions[0]);

			// ejecutamos la accion
			commissioningMgr.executeAction(commissioning, workflowActions[0]);
			// como estamos con el pedido con id=2
			// NO debe pasar la comprobacin de stock, por lo
			// que estaremos en el paso con id=2
			Assert.assertEquals(2, commissioning.getState().intValue());
			// si ahora recuperamos las acciones disponibles
			// slo debemos tener una, la accin de solicitar al distribuidor (id=2)
			workflowActions = commissioningMgr.getAvalaibleActions(commissioning);
			Assert.assertEquals(1, workflowActions.length);
			Assert.assertEquals(2, workflowActions[0]);

			// ejecutamos la accin disponible es solicitar (id=2)
			commissioningMgr.executeAction(commissioning, workflowActions[0]);

			// estamos en el paso 3
			Assert.assertEquals(3, commissioning.getState().intValue());
			// si ahora recuperamos las acciones disponibles
			// debemos tener 2, la accin de recibir (id=3) y
			// la accion de cancelar (id=4)
			workflowActions = commissioningMgr.getAvalaibleActions(commissioning);
			Assert.assertEquals(2, workflowActions.length);
			if ((workflowActions[0] != 3) && (workflowActions[0] != 4)) {
				Assert.fail("Accin disponible no esperada");
			}
			if ((workflowActions[1] != 3) && (workflowActions[1] != 4)) {
				Assert.fail("Accin disponible no esperada");
			}

			// ejecutamos la accin disponible de recibir (id=3)
			commissioningMgr.executeAction(commissioning, 3);

			// ahora ha vuelto al paso 1
			Assert.assertEquals(1, commissioning.getState().intValue());
			// comprobamos que hemos vuelto al primer paso
			// teniendo nicamente disponible la accin depreparar
			workflowActions = commissioningMgr.getAvalaibleActions(commissioning);
			// slo debemos tener la accion de preparar
			Assert.assertEquals(1, workflowActions.length);
			Assert.assertEquals(1, workflowActions[0]);

			// ahora volvemos a ejecutar los pasos que hemos probado
			// antes para probar la cancelacin y terminar

			// preparar
			commissioningMgr.executeAction(commissioning, 1);
			Assert.assertEquals(2, commissioning.getState().intValue());
			// solicitar distribucin
			commissioningMgr.executeAction(commissioning, 2);
			Assert.assertEquals(3, commissioning.getState().intValue());
			// cancelar
			commissioningMgr.executeAction(commissioning, 4);
			// ahora ha se ha ido al paso 4 (estado de cancelado)
			Assert.assertEquals(4, commissioning.getState().intValue());
			// que no tiene ninguna accin
			workflowActions = commissioningMgr.getAvalaibleActions(commissioning);
			Assert.assertEquals(0, workflowActions.length);

		} catch (final MyAppException e) {
			e.printStackTrace();
			Assert.fail("Error");
		}
	}
}
